@@ -1,6 +1,6 @@
 # Dynamic architecture support for GDB, the GNU debugger.
 
-# Copyright (C) 1998-2025 Free Software Foundation, Inc.
+# Copyright (C) 1998-2026 Free Software Foundation, Inc.
 
 # This file is part of GDB.
 
@@ -544,39 +544,6 @@ Value(
     invalid=False,
 )
 
-Method(
-    comment="""
-Convert stab register number (from `r' declaration) to a gdb REGNUM.
-""",
-    type="int",
-    name="stab_reg_to_regnum",
-    params=[("int", "stab_regnr")],
-    predefault="no_op_reg_to_regnum",
-    invalid=False,
-)
-
-Method(
-    comment="""
-Provide a default mapping from a ecoff register number to a gdb REGNUM.
-""",
-    type="int",
-    name="ecoff_reg_to_regnum",
-    params=[("int", "ecoff_regnr")],
-    predefault="no_op_reg_to_regnum",
-    invalid=False,
-)
-
-Method(
-    comment="""
-Convert from an sdb register number to an internal gdb register number.
-""",
-    type="int",
-    name="sdb_reg_to_regnum",
-    params=[("int", "sdb_regnr")],
-    predefault="no_op_reg_to_regnum",
-    invalid=False,
-)
-
 Method(
     comment="""
 Provide a default mapping from a DWARF2 register number to a gdb REGNUM.
@@ -930,7 +897,7 @@ Function(
     comment="""
 Return the address at which the value being returned from
 the current function will be stored.  This routine is only
-called if the current function uses the the "struct return
+called if the current function uses the "struct return
 convention".
 
 May return 0 when unable to determine that address.""",
@@ -1201,14 +1168,6 @@ Method(
     predicate=True,
 )
 
-Method(
-    type="int",
-    name="stabs_argument_has_addr",
-    params=[("struct type *", "type")],
-    predefault="default_stabs_argument_has_addr",
-    invalid=False,
-)
-
 Value(
     type="int",
     name="frame_red_zone_size",
