@@ -1,6 +1,6 @@
 /* Top level stuff for GDB, the GNU debugger.
 
-   Copyright (C) 1986-2025 Free Software Foundation, Inc.
+   Copyright (C) 1986-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -69,7 +69,6 @@
 
 #include "event-top.h"
 #include <sys/stat.h>
-#include <ctype.h>
 #include "ui-out.h"
 #include "cli-out.h"
 #include "tracepoint.h"
@@ -468,8 +467,7 @@ execute_command (const char *p, int from_tty)
 
   target_log_command (p);
 
-  while (*p == ' ' || *p == '\t')
-    p++;
+  p = skip_spaces (p);
   if (*p)
     {
       const char *cmd = p;
@@ -1312,7 +1310,7 @@ print_gdb_version (struct ui_file *stream, bool interactive)
   /* Second line is a copyright notice.  */
 
   gdb_printf (stream,
-	      "Copyright (C) 2025 Free Software Foundation, Inc.\n");
+	      "Copyright (C) 2026 Free Software Foundation, Inc.\n");
 
   /* Following the copyright is a brief statement that the program is
      free software, that users are free to copy and change it on
@@ -1358,17 +1356,162 @@ There is NO WARRANTY, to the extent permitted by law.",
 		  styled_string (file_name_style.style (),
 				 REPORT_BUGS_TO));
     }
-  gdb_printf (stream,
-	      _("Find the GDB manual and other documentation \
-resources online at:\n    <%ps>."),
-	      styled_string (file_name_style.style (),
-			     "http://www.gnu.org/software/gdb/documentation/"));
-  gdb_printf (stream, "\n\n");
-  gdb_printf (stream, _("For help, type \"%ps\".\n"),
+}
+
+/* Unicode Block “Box Drawing” chars.  UTF-8 string literals have type:
+   - const char[N]    (until C++20), or
+   - const char8_t[N] (since C++20).
+   Assign them to a variable to stabilize the type.
+*/
+static const char bd_heavy_horizontal[] = u8"\u2501";
+static const char bd_heavy_vertical[] = u8"\u2503";
+static const char bd_heavy_down_and_right[] = u8"\u250f";
+static const char bd_heavy_down_and_left[] = u8"\u2513";
+static const char bd_heavy_up_and_right[] = u8"\u2517";
+static const char bd_heavy_up_and_left[] = u8"\u251b";
+
+/* Print MESSAGE to STREAM in lines of maximum size WIDTH, so that it fits
+   in an ascii art box of width WIDTH+4.  Messages may be broken on
+   spaces.  */
+static void
+box_one_message (ui_file *stream, std::string message, int width)
+{
+  const char *wall = emojis_ok () ? bd_heavy_vertical : "|";
+
+  /* We know that the box we are displaying the text in is at least as
+     long as the documentation URL.
+
+     Additionally, we know that on non-URL lines, any styling occurs in the
+     first part of the line, before we might need to line wrap.  We take
+     advantage of this to simplify this function slightly.
+
+     First, count the total number of escape characters on this line.  */
+  int n_escape_chars = 0;
+  const char *escape = message.c_str ();
+  while ((escape = strchr (escape, '\033')) != nullptr)
+    {
+      int tmp;
+      if (skip_ansi_escape (escape, &tmp))
+	n_escape_chars += tmp;
+      else
+	gdb_assert_not_reached ("unknown escape sequence");
+      escape += tmp;
+    }
+
+  /* ESCAPE points to the first character after the last escape sequence in
+     MESSAGE.  The number of non-escape sequence characters up to this
+     point should always be less than width.  If this fails then we need to
+     wrap the escape sequence block, and this function isn't written with
+     that in mind.  */
+  gdb_assert ((escape - message.c_str () - n_escape_chars) < width);
+
+  while (!message.empty ())
+    {
+      std::string line;
+      if ((message.length () - n_escape_chars) > width)
+	{
+	  /* Place the left hand part of MESSAGE into LINE.  Take account
+	     of any escape characters in MESSAGE.  */
+	  line = message.substr (0, message.rfind (' ',
+						   width + n_escape_chars));
+
+	  /* The '+ 1' here skips the space that rfind just found.  */
+	  message = message.substr (line.length () + 1);
+	}
+      else
+	line = std::move (message);
+
+      /* Pad LINE with spaces so that the right border is in the correct
+	 place.  */
+      if ((line.length () - n_escape_chars) < width)
+	line.append (width - line.length () + n_escape_chars, ' ');
+
+      gdb_printf (stream, "%s %s %s\n", wall, line.c_str (), wall);
+
+      /* As we know all escape characters fall in the first part of the
+	 line (before any wrapping), then after printing the first part of
+	 the line (which might be all of the line), we know there are no
+	 remaining escape characters.  If there were then the assert above
+	 this loop would have triggered.  */
+      n_escape_chars = 0;
+    }
+}
+
+/* Print some hints about how to use GDB in a very visible manner.  */
+void
+print_gdb_hints (struct ui_file *stream)
+{
+  unsigned int width = get_chars_per_line ();
+
+  /* Arbitrarily setting maximum width to 80 characters, so that
+     things are big and visible but not overwhelming.  */
+  if (80 < width)
+    width = 80;
+
+  gdb_assert (width > 0);
+
+  std::string docs_url = "http://www.gnu.org/software/gdb/documentation/";
+  std::array<string_file, 4> styled_msg {
+    string_file (true),
+    string_file (true),
+    string_file (true),
+    string_file (true)
+  };
+
+  gdb_printf (&styled_msg[0], _("Find the GDB manual online at:"));
+  gdb_printf (&styled_msg[1], _("%ps."),
+	      styled_string (file_name_style.style (), docs_url.c_str ()));
+  gdb_printf (&styled_msg[2], _("For help, type \"%ps\"."),
 	      styled_string (command_style.style (), "help"));
-  gdb_printf (stream,
+  gdb_printf (&styled_msg[3],
 	      _("Type \"%ps\" to search for commands related to \"word\"."),
 	      styled_string (command_style.style (), "apropos word"));
+
+  /* If there isn't enough space to display the longest URL in a boxed
+     style, then don't use the box, the terminal will break the output
+     where needed.  The longest URL is used because the other messages may
+     be broken into multiple lines, but URLs can't.
+
+     The ' + 1' after the URL accounts for the period that is placed after
+     the URL.
+
+     The '+ 4' accounts for the box and inner white space.  We add the 4 to
+     the string length rather than subtract from the width as the width
+     could be less than 4, and we want to avoid wrap around.  */
+  if (width < docs_url.length () + 1 + 4)
+    {
+      for (string_file &msg : styled_msg)
+	gdb_printf (stream, "%s\n", msg.c_str ());
+    }
+  else
+    {
+      gdb_assert (width > 4);
+
+      std::string sep (width - 2, '-');
+
+      if (emojis_ok ())
+	{
+	  gdb_printf (stream, "%s", bd_heavy_down_and_right);
+	  for (int i = 0; i < (width - 2); i++)
+	    gdb_printf (stream, "%s", bd_heavy_horizontal);
+	  gdb_printf (stream, "%s\n", bd_heavy_down_and_left);
+	}
+      else
+	gdb_printf (stream, "+%s+\n", sep.c_str ());
+
+      for (string_file &msg : styled_msg)
+	box_one_message (stream, msg.release (), width - 4);
+
+      if (emojis_ok ())
+	{
+	  gdb_printf (stream, "%s", bd_heavy_up_and_right);
+	  for (int i = 0; i < (width - 2); i++)
+	    gdb_printf (stream, "%s", bd_heavy_horizontal);
+	  gdb_printf (stream, "%s\n", bd_heavy_up_and_left);
+	}
+      else
+	gdb_printf (stream, "+%s+\n", sep.c_str ());
+    }
 }
 
 /* Print the details of GDB build-time configuration.  */
@@ -1576,7 +1719,7 @@ This GDB was configured as follows:\n\
 "), DEBUGDIR, DEBUGDIR_RELOCATABLE ? " (relocatable)" : "");
 
 #ifdef ADDITIONAL_DEBUG_DIRS
-  gdb_printf (stream, _ ("\
+  gdb_printf (stream, _("\
 	     --with-additional-debug-dirs=%s\n\
 "), ADDITIONAL_DEBUG_DIRS);
 #endif
@@ -1725,6 +1868,10 @@ undo_terminal_modifications_before_exit (void)
 #endif
   gdb_disable_readline ();
 
+#ifdef __MINGW32__
+  mingw_deinitialize_console ();
+#endif
+
   current_ui = saved_top_level;
 }
 
