@@ -1,4 +1,4 @@
-/* Copyright (C) 2023-2025 Free Software Foundation, Inc.
+/* Copyright (C) 2023-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -227,7 +227,9 @@ using ui_range = next_range<ui>;
 static inline
 ui_range all_uis ()
 {
-  return ui_range (ui_list);
+  next_iterator<ui> begin (ui_list);
+
+  return ui_range (std::move (begin));
 }
 
 #endif /* GDB_UI_H */
