@@ -1,6 +1,6 @@
 /* XML target description support for GDB.
 
-   Copyright (C) 2006-2017 Free Software Foundation, Inc.
+   Copyright (C) 2006-2026 Free Software Foundation, Inc.
 
    Contributed by CodeSourcery.
 
@@ -19,14 +19,13 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "target.h"
 #include "target-descriptions.h"
 #include "xml-support.h"
 #include "xml-tdesc.h"
 #include "osabi.h"
 #include "filenames.h"
-#include <unordered_map>
+#include "gdbsupport/unordered_map.h"
 #include <string>
 
 /* Maximum sizes.
@@ -41,8 +40,7 @@
    an XML parser.  */
 
 static struct target_desc *
-tdesc_parse_xml (const char *document, xml_fetch_another fetcher,
-		 void *fetcher_baton)
+tdesc_parse_xml (const char *document, xml_fetch_another fetcher)
 {
   static int have_warned;
 
@@ -66,7 +64,7 @@ tdesc_parse_xml (const char *document, xml_fetch_another fetcher,
    then we will create unnecessary duplicate gdbarches.  See
    gdbarch_list_lookup_by_info.  */
 
-static std::unordered_map<std::string, target_desc *> xml_cache;
+static gdb::unordered_map<std::string, target_desc_up> xml_cache;
 
 /* Callback data for target description parsing.  */
 
@@ -144,9 +142,10 @@ tdesc_end_compatible (struct gdb_xml_parser *parser,
 static void
 tdesc_start_target (struct gdb_xml_parser *parser,
 		    const struct gdb_xml_element *element,
-		    void *user_data, VEC(gdb_xml_value_s) *attributes)
+		    void *user_data, std::vector<gdb_xml_value> &attributes)
 {
-  char *version = (char *) xml_find_attribute (attributes, "version")->value;
+  char *version
+    = (char *) xml_find_attribute (attributes, "version")->value.get ();
 
   if (strcmp (version, "1.0") != 0)
     gdb_xml_error (parser,
@@ -159,10 +158,10 @@ tdesc_start_target (struct gdb_xml_parser *parser,
 static void
 tdesc_start_feature (struct gdb_xml_parser *parser,
 		     const struct gdb_xml_element *element,
-		     void *user_data, VEC(gdb_xml_value_s) *attributes)
+		     void *user_data, std::vector<gdb_xml_value> &attributes)
 {
   struct tdesc_parsing_data *data = (struct tdesc_parsing_data *) user_data;
-  char *name = (char *) xml_find_attribute (attributes, "name")->value;
+  char *name = (char *) xml_find_attribute (attributes, "name")->value.get ();
 
   data->current_feature = tdesc_create_feature (data->tdesc, name);
 }
@@ -173,37 +172,36 @@ tdesc_start_feature (struct gdb_xml_parser *parser,
 static void
 tdesc_start_reg (struct gdb_xml_parser *parser,
 		 const struct gdb_xml_element *element,
-		 void *user_data, VEC(gdb_xml_value_s) *attributes)
+		 void *user_data, std::vector<gdb_xml_value> &attributes)
 {
   struct tdesc_parsing_data *data = (struct tdesc_parsing_data *) user_data;
-  struct gdb_xml_value *attrs = VEC_address (gdb_xml_value_s, attributes);
-  int ix = 0, length;
+  int ix = 0;
   char *name, *group;
   const char *type;
   int bitsize, regnum, save_restore;
 
-  length = VEC_length (gdb_xml_value_s, attributes);
+  int length = attributes.size ();
 
-  name = (char *) attrs[ix++].value;
-  bitsize = * (ULONGEST *) attrs[ix++].value;
+  name = (char *) attributes[ix++].value.get ();
+  bitsize = * (ULONGEST *) attributes[ix++].value.get ();
 
-  if (ix < length && strcmp (attrs[ix].name, "regnum") == 0)
-    regnum = * (ULONGEST *) attrs[ix++].value;
+  if (ix < length && strcmp (attributes[ix].name, "regnum") == 0)
+    regnum = * (ULONGEST *) attributes[ix++].value.get ();
   else
     regnum = data->next_regnum;
 
-  if (ix < length && strcmp (attrs[ix].name, "type") == 0)
-    type = (char *) attrs[ix++].value;
+  if (ix < length && strcmp (attributes[ix].name, "type") == 0)
+    type = (char *) attributes[ix++].value.get ();
   else
     type = "int";
 
-  if (ix < length && strcmp (attrs[ix].name, "group") == 0)
-    group = (char *) attrs[ix++].value;
+  if (ix < length && strcmp (attributes[ix].name, "group") == 0)
+    group = (char *) attributes[ix++].value.get ();
   else
     group = NULL;
 
-  if (ix < length && strcmp (attrs[ix].name, "save-restore") == 0)
-    save_restore = * (ULONGEST *) attrs[ix++].value;
+  if (ix < length && strcmp (attributes[ix].name, "save-restore") == 0)
+    save_restore = * (ULONGEST *) attributes[ix++].value.get ();
   else
     save_restore = 1;
 
@@ -225,10 +223,10 @@ tdesc_start_reg (struct gdb_xml_parser *parser,
 static void
 tdesc_start_union (struct gdb_xml_parser *parser,
 		   const struct gdb_xml_element *element,
-		   void *user_data, VEC(gdb_xml_value_s) *attributes)
+		   void *user_data, std::vector<gdb_xml_value> &attributes)
 {
   struct tdesc_parsing_data *data = (struct tdesc_parsing_data *) user_data;
-  char *id = (char *) xml_find_attribute (attributes, "id")->value;
+  char *id = (char *) xml_find_attribute (attributes, "id")->value.get ();
 
   data->current_type = tdesc_create_union (data->current_feature, id);
   data->current_type_size = 0;
@@ -240,10 +238,10 @@ tdesc_start_union (struct gdb_xml_parser *parser,
 static void
 tdesc_start_struct (struct gdb_xml_parser *parser,
 		   const struct gdb_xml_element *element,
-		   void *user_data, VEC(gdb_xml_value_s) *attributes)
+		   void *user_data, std::vector<gdb_xml_value> &attributes)
 {
   struct tdesc_parsing_data *data = (struct tdesc_parsing_data *) user_data;
-  char *id = (char *) xml_find_attribute (attributes, "id")->value;
+  char *id = (char *) xml_find_attribute (attributes, "id")->value.get ();
   struct gdb_xml_value *attr;
 
   tdesc_type_with_fields *type_with_fields
@@ -254,7 +252,7 @@ tdesc_start_struct (struct gdb_xml_parser *parser,
   attr = xml_find_attribute (attributes, "size");
   if (attr != NULL)
     {
-      ULONGEST size = * (ULONGEST *) attr->value;
+      ULONGEST size = * (ULONGEST *) attr->value.get ();
 
       if (size > MAX_FIELD_SIZE)
 	{
@@ -270,12 +268,12 @@ tdesc_start_struct (struct gdb_xml_parser *parser,
 static void
 tdesc_start_flags (struct gdb_xml_parser *parser,
 		   const struct gdb_xml_element *element,
-		   void *user_data, VEC(gdb_xml_value_s) *attributes)
+		   void *user_data, std::vector<gdb_xml_value> &attributes)
 {
   struct tdesc_parsing_data *data = (struct tdesc_parsing_data *) user_data;
-  char *id = (char *) xml_find_attribute (attributes, "id")->value;
+  char *id = (char *) xml_find_attribute (attributes, "id")->value.get ();
   ULONGEST size = * (ULONGEST *)
-    xml_find_attribute (attributes, "size")->value;
+    xml_find_attribute (attributes, "size")->value.get ();
 
   if (size > MAX_FIELD_SIZE)
     {
@@ -291,12 +289,12 @@ tdesc_start_flags (struct gdb_xml_parser *parser,
 static void
 tdesc_start_enum (struct gdb_xml_parser *parser,
 		  const struct gdb_xml_element *element,
-		  void *user_data, VEC(gdb_xml_value_s) *attributes)
+		  void *user_data, std::vector<gdb_xml_value> &attributes)
 {
   struct tdesc_parsing_data *data = (struct tdesc_parsing_data *) user_data;
-  char *id = (char *) xml_find_attribute (attributes, "id")->value;
+  char *id = (char *) xml_find_attribute (attributes, "id")->value.get ();
   int size = * (ULONGEST *)
-    xml_find_attribute (attributes, "size")->value;
+    xml_find_attribute (attributes, "size")->value.get ();
 
   if (size > MAX_FIELD_SIZE)
     {
@@ -315,7 +313,7 @@ tdesc_start_enum (struct gdb_xml_parser *parser,
 static void
 tdesc_start_field (struct gdb_xml_parser *parser,
 		   const struct gdb_xml_element *element,
-		   void *user_data, VEC(gdb_xml_value_s) *attributes)
+		   void *user_data, std::vector<gdb_xml_value> &attributes)
 {
   struct tdesc_parsing_data *data = (struct tdesc_parsing_data *) user_data;
   struct gdb_xml_value *attr;
@@ -323,12 +321,12 @@ tdesc_start_field (struct gdb_xml_parser *parser,
   char *field_name, *field_type_id;
   int start, end;
 
-  field_name = (char *) xml_find_attribute (attributes, "name")->value;
+  field_name = (char *) xml_find_attribute (attributes, "name")->value.get ();
 
   attr = xml_find_attribute (attributes, "type");
   if (attr != NULL)
     {
-      field_type_id = (char *) attr->value;
+      field_type_id = (char *) attr->value.get ();
       field_type = tdesc_named_type (data->current_feature, field_type_id);
     }
   else
@@ -340,7 +338,7 @@ tdesc_start_field (struct gdb_xml_parser *parser,
   attr = xml_find_attribute (attributes, "start");
   if (attr != NULL)
     {
-      ULONGEST ul_start = * (ULONGEST *) attr->value;
+      ULONGEST ul_start = * (ULONGEST *) attr->value.get ();
 
       if (ul_start > MAX_FIELD_BITSIZE)
 	{
@@ -356,7 +354,7 @@ tdesc_start_field (struct gdb_xml_parser *parser,
   attr = xml_find_attribute (attributes, "end");
   if (attr != NULL)
     {
-      ULONGEST ul_end = * (ULONGEST *) attr->value;
+      ULONGEST ul_end = * (ULONGEST *) attr->value.get ();
 
       if (ul_end > MAX_FIELD_BITSIZE)
 	{
@@ -374,7 +372,7 @@ tdesc_start_field (struct gdb_xml_parser *parser,
       tdesc_type_with_fields *t = data->current_type;
 
       /* Older versions of gdb can't handle elided end values.
-         Stick with that for now, to help ensure backward compatibility.
+	 Stick with that for now, to help ensure backward compatibility.
 	 E.g., If a newer gdbserver is talking to an older gdb.  */
       if (end == -1)
 	gdb_xml_error (parser, _("Missing end value"));
@@ -444,7 +442,7 @@ tdesc_start_field (struct gdb_xml_parser *parser,
 static void
 tdesc_start_enum_value (struct gdb_xml_parser *parser,
 			const struct gdb_xml_element *element,
-			void *user_data, VEC(gdb_xml_value_s) *attributes)
+			void *user_data, std::vector<gdb_xml_value> &attributes)
 {
   struct tdesc_parsing_data *data = (struct tdesc_parsing_data *) user_data;
   struct gdb_xml_value *attr;
@@ -452,10 +450,10 @@ tdesc_start_enum_value (struct gdb_xml_parser *parser,
   ULONGEST ul_value;
   int value;
 
-  field_name = (char *) xml_find_attribute (attributes, "name")->value;
+  field_name = (char *) xml_find_attribute (attributes, "name")->value.get ();
 
   attr = xml_find_attribute (attributes, "value");
-  ul_value = * (ULONGEST *) attr->value;
+  ul_value = * (ULONGEST *) attr->value.get ();
   if (ul_value > INT_MAX)
     {
       gdb_xml_error (parser,
@@ -473,17 +471,16 @@ tdesc_start_enum_value (struct gdb_xml_parser *parser,
 static void
 tdesc_start_vector (struct gdb_xml_parser *parser,
 		    const struct gdb_xml_element *element,
-		    void *user_data, VEC(gdb_xml_value_s) *attributes)
+		    void *user_data, std::vector<gdb_xml_value> &attributes)
 {
   struct tdesc_parsing_data *data = (struct tdesc_parsing_data *) user_data;
-  struct gdb_xml_value *attrs = VEC_address (gdb_xml_value_s, attributes);
   struct tdesc_type *field_type;
   char *id, *field_type_id;
   ULONGEST count;
 
-  id = (char *) attrs[0].value;
-  field_type_id = (char *) attrs[1].value;
-  count = * (ULONGEST *) attrs[2].value;
+  id = (char *) attributes[0].value.get ();
+  field_type_id = (char *) attributes[1].value.get ();
+  count = * (ULONGEST *) attributes[2].value.get ();
 
   if (count > MAX_VECTOR_SIZE)
     {
@@ -581,7 +578,7 @@ static const struct gdb_xml_element feature_children[] = {
     tdesc_start_union, NULL },
   { "flags", flags_attributes, struct_union_children,
     GDB_XML_EF_OPTIONAL | GDB_XML_EF_REPEATABLE,
-    tdesc_start_flags, NULL },    
+    tdesc_start_flags, NULL },
   { "enum", enum_attributes, enum_children,
     GDB_XML_EF_OPTIONAL | GDB_XML_EF_REPEATABLE,
     tdesc_start_enum, NULL },
@@ -618,8 +615,7 @@ static const struct gdb_xml_element tdesc_elements[] = {
 /* Parse DOCUMENT into a target description and return it.  */
 
 static struct target_desc *
-tdesc_parse_xml (const char *document, xml_fetch_another fetcher,
-		 void *fetcher_baton)
+tdesc_parse_xml (const char *document, xml_fetch_another fetcher)
 {
   struct tdesc_parsing_data data;
 
@@ -628,7 +624,7 @@ tdesc_parse_xml (const char *document, xml_fetch_another fetcher,
 
   if (!xml_process_xincludes (expanded_text,
 			      _("target description"),
-			      document, fetcher, fetcher_baton, 0))
+			      document, fetcher, 0))
     {
       warning (_("Could not load XML target description; ignoring"));
       return NULL;
@@ -638,25 +634,22 @@ tdesc_parse_xml (const char *document, xml_fetch_another fetcher,
      previously parsed.  */
   const auto it = xml_cache.find (expanded_text);
   if (it != xml_cache.end ())
-    return it->second;
+    return it->second.get ();
 
   memset (&data, 0, sizeof (struct tdesc_parsing_data));
-  data.tdesc = allocate_target_description ();
-  struct cleanup *result_cleanup
-    = make_cleanup_free_target_description (data.tdesc);
+  target_desc_up description = allocate_target_description ();
+  data.tdesc = description.get ();
 
   if (gdb_xml_parse_quick (_("target description"), "gdb-target.dtd",
 			   tdesc_elements, expanded_text.c_str (), &data) == 0)
     {
       /* Parsed successfully.  */
-      xml_cache.emplace (std::move (expanded_text), data.tdesc);
-      discard_cleanups (result_cleanup);
+      xml_cache.emplace (std::move (expanded_text), std::move (description));
       return data.tdesc;
     }
   else
     {
       warning (_("Could not load XML target description; ignoring"));
-      do_cleanups (result_cleanup);
       return NULL;
     }
 }
@@ -669,16 +662,21 @@ tdesc_parse_xml (const char *document, xml_fetch_another fetcher,
 const struct target_desc *
 file_read_description_xml (const char *filename)
 {
-  gdb::unique_xmalloc_ptr<char> tdesc_str
+  std::optional<gdb::char_vector> tdesc_str
     = xml_fetch_content_from_file (filename, NULL);
-  if (tdesc_str == NULL)
+  if (!tdesc_str)
     {
       warning (_("Could not open \"%s\""), filename);
       return NULL;
     }
 
-  return tdesc_parse_xml (tdesc_str.get (), xml_fetch_content_from_file,
-			  (void *) ldirname (filename).c_str ());
+  const std::string dirname = gdb_ldirname (filename);
+  auto fetch_another = [&dirname] (const char *name)
+    {
+      return xml_fetch_content_from_file (name, dirname.c_str ());
+    };
+
+  return tdesc_parse_xml (tdesc_str->data (), fetch_another);
 }
 
 /* Read a string representation of available features from the target,
@@ -688,11 +686,9 @@ file_read_description_xml (const char *filename)
    is "target.xml".  Other calls may be performed for the DTD or
    for <xi:include>.  */
 
-static gdb::unique_xmalloc_ptr<char>
-fetch_available_features_from_target (const char *name, void *baton_)
+static std::optional<gdb::char_vector>
+fetch_available_features_from_target (const char *name, target_ops *ops)
 {
-  struct target_ops *ops = (struct target_ops *) baton_;
-
   /* Read this object as a string.  This ensures that a NUL
      terminator is added.  */
   return target_read_stralloc (ops,
@@ -707,21 +703,24 @@ fetch_available_features_from_target (const char *name, void *baton_)
 const struct target_desc *
 target_read_description_xml (struct target_ops *ops)
 {
-  gdb::unique_xmalloc_ptr<char> tdesc_str
+  std::optional<gdb::char_vector> tdesc_str
     = fetch_available_features_from_target ("target.xml", ops);
-  if (tdesc_str == NULL)
+  if (!tdesc_str)
     return NULL;
 
-  return tdesc_parse_xml (tdesc_str.get (),
-			  fetch_available_features_from_target,
-			  ops);
+  auto fetch_another = [ops] (const char *name)
+    {
+      return fetch_available_features_from_target (name, ops);
+    };
+
+  return tdesc_parse_xml (tdesc_str->data (), fetch_another);
 }
 
 /* Fetches an XML target description using OPS,  processing
    includes, but not parsing it.  Used to dump whole tdesc
    as a single XML file.  */
 
-gdb::optional<std::string>
+std::optional<std::string>
 target_fetch_description_xml (struct target_ops *ops)
 {
 #if !defined(HAVE_LIBEXPAT)
@@ -736,16 +735,19 @@ target_fetch_description_xml (struct target_ops *ops)
 
   return {};
 #else
-  gdb::unique_xmalloc_ptr<char>
+  std::optional<gdb::char_vector>
     tdesc_str = fetch_available_features_from_target ("target.xml", ops);
-  if (tdesc_str == NULL)
+  if (!tdesc_str)
     return {};
 
+  auto fetch_another = [ops] (const char *name)
+    {
+      return fetch_available_features_from_target (name, ops);
+    };
   std::string output;
   if (!xml_process_xincludes (output,
 			      _("target description"),
-			      tdesc_str.get (),
-			      fetch_available_features_from_target, ops, 0))
+			      tdesc_str->data (), fetch_another, 0))
     {
       warning (_("Could not load XML target description; ignoring"));
       return {};
@@ -753,3 +755,15 @@ target_fetch_description_xml (struct target_ops *ops)
   return output;
 #endif
 }
+
+/* See xml-tdesc.h.  */
+
+const struct target_desc *
+string_read_description_xml (const char *xml)
+{
+  return tdesc_parse_xml (xml, [] (const char *href)
+    {
+      error (_("xincludes are unsupported with this method"));
+      return std::optional<gdb::char_vector> ();
+    });
+}
