@@ -1,5 +1,5 @@
 /* Internal interfaces for the Win32 specific target code for gdbserver.
-   Copyright (C) 2007-2025 Free Software Foundation, Inc.
+   Copyright (C) 2007-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -20,18 +20,12 @@
 #define GDBSERVER_WIN32_LOW_H
 
 #include <windows.h>
+#include "gdbsupport/tdesc.h"
 #include "nat/windows-nat.h"
 #include "gdbsupport/osabi.h"
 
 struct target_desc;
 
-/* The inferior's target description.  This is a global because the
-   Windows ports support neither bi-arch nor multi-process.  */
-extern const struct target_desc *win32_tdesc;
-#ifdef __x86_64__
-extern const struct target_desc *wow64_win32_tdesc;
-#endif
-
 #ifdef __CYGWIN__
 constexpr enum gdb_osabi WINDOWS_OSABI = GDB_OSABI_CYGWIN;
 #else
@@ -47,7 +41,7 @@ struct win32_target_ops
   int (*num_regs) (void);
 
   /* Perform initializations on startup.  */
-  void (*initial_stuff) (void);
+  void (*initial_stuff) (process_info *proc);
 
   /* Fetch the context from the inferior.  */
   void (*get_thread_context) (windows_nat::windows_thread_info *th);
@@ -89,7 +83,10 @@ struct win32_target_ops
   int (*remove_point) (enum raw_bkpt_type type, CORE_ADDR addr,
 		       int size, struct raw_breakpoint *bp);
   int (*stopped_by_watchpoint) (void);
-  CORE_ADDR (*stopped_data_address) (void);
+  std::vector<CORE_ADDR> (*stopped_data_addresses) ();
+
+  /* Determine if ER contains a software-breakpoint.  */
+  bool (*is_sw_breakpoint) (const EXCEPTION_RECORD *er);
 };
 
 extern struct win32_target_ops the_low_target;
@@ -144,7 +141,7 @@ public:
 
   bool stopped_by_watchpoint () override;
 
-  CORE_ADDR stopped_data_address () override;
+  std::vector<CORE_ADDR> stopped_data_addresses () override;
 
   bool supports_qxfer_siginfo () override;
 
